unit FMX.SpinBox.iOS;

interface

{$SCOPEDENUMS ON}

uses
  System.Types, System.TypInfo, iOSapi.UIKit, iOSapi.Foundation, FMX.Controls.Presentation, FMX.Presentation.iOS,
  FMX.Presentation.Messages, FMX.EditBox, FMX.SpinBox, FMX.Types;

type

{ TiOSNativeSpinBox }

  IFMXUIStepper = interface(UIStepper)
  ['{460C2A47-6D76-4945-BCBF-4A14B2147C48}']
    procedure ValueChanged; cdecl;
    { Touches }
    procedure touchesBegan(touches: NSSet; withEvent: UIEvent); cdecl;
    procedure touchesCancelled(touches: NSSet; withEvent: UIEvent); cdecl;
    procedure touchesEnded(touches: NSSet; withEvent: UIEvent); cdecl;
    procedure touchesMoved(touches: NSSet; withEvent: UIEvent); cdecl;
  end;

  TiOSNativeSpinBox = class(TiOSNativeControl)
  private
    [Weak] FModel: TSpinBoxModel;
    function GetView: UIStepper;
  protected
    function GetObjectiveCClass: PTypeInfo; override;
  protected
    { Messages from Model }
    procedure MMRepeatClickChanged(var AMessage: TDispatchMessage); message MM_SPINBOX_REPEATCLICK_CHANGED;
    procedure MMValueRangeChanged(var AMessage: TDispatchMessage); message MM_VALUERANGE_CHANGED;
    { Messages from PresentationProxy }
    procedure PMSetModel(var AMessage: TDispatchMessageWithValue<TMessageSender>); message PM_SET_MODEL;
    procedure PMInit(var AMessage: TDispatchMessage); message PM_INIT;
    procedure PMGetAdjustSize(var AMessage: TDispatchMessageWithValue<TSizeF>); message PM_GET_ADJUST_SIZE;
    procedure PMGetAdjustType(var AMessage: TDispatchMessageWithValue<TAdjustType>); message PM_GET_ADJUST_TYPE;
  public
    constructor Create; override;
    procedure ValueChanged; cdecl;
  public
    property Model: TSpinBoxModel read FModel;
    property View: UIStepper read GetView;
  end;

{ TiOSSpinBoxProxy }

  TiOSSpinBoxProxy = class(TPresentationProxy)
  protected
    function CreateReceiver: TObject; override;
  end;

implementation

uses
  System.Classes, System.SysUtils, Macapi.ObjCRuntime, FMX.Controls, FMX.Presentation.Factory, FMX.Consts;

{ TiOSSpinBoxProxy }

function TiOSSpinBoxProxy.CreateReceiver: TObject;
begin
  Result := TiOSNativeSpinBox.Create;
end;

{ TiOSNativeSpinBox }

constructor TiOSNativeSpinBox.Create;
begin
  inherited;
  View.setWraps(False);
  View.setContinuous(True);
  RegisterNativeEventHandler('ValueChanged', UIControlEventValueChanged);
end;

function TiOSNativeSpinBox.GetObjectiveCClass: PTypeInfo;
begin
  Result := TypeInfo(IFMXUIStepper);
end;

function TiOSNativeSpinBox.GetView: UIStepper;
begin
  Result := inherited GetView<UIStepper>;
end;

procedure TiOSNativeSpinBox.MMRepeatClickChanged(var AMessage: TDispatchMessage);
begin
  View.setAutorepeat(Model.RepeatClick);
end;

procedure TiOSNativeSpinBox.MMValueRangeChanged(var AMessage: TDispatchMessage);
begin
  View.setValue(Model.Value);
  View.setMinimumValue(Model.ValueRange.Min);
  View.setMaximumValue(Model.ValueRange.Max);
end;

procedure TiOSNativeSpinBox.PMGetAdjustSize(var AMessage: TDispatchMessageWithValue<TSizeF>);
begin
  AMessage.Value := TSizeF.Create(View.frame.size.width, View.frame.size.height);
end;

procedure TiOSNativeSpinBox.PMGetAdjustType(var AMessage: TDispatchMessageWithValue<TAdjustType>);
begin
  AMessage.Value := TAdjustType.FixedSize;
end;

procedure TiOSNativeSpinBox.PMInit(var AMessage: TDispatchMessage);
begin
  View.setValue(Model.Value);
  View.setMinimumValue(Model.ValueRange.Min);
  View.setMaximumValue(Model.ValueRange.Max);
  View.setAutorepeat(Model.RepeatClick);
  View.setStepValue(Model.HorzIncrement);
end;

procedure TiOSNativeSpinBox.PMSetModel(var AMessage: TDispatchMessageWithValue<TMessageSender>);
begin
  if AMessage.Value is TSpinBoxModel then
  begin
    FModel := TSpinBoxModel(AMessage.Value);
    FModel.Receiver := Self;
  end
  else
    raise EPresentationWrongModel.Create(Format(SWrongModelClassType, [TSpinBoxModel.ClassName, AMessage.Value.ClassName]));
end;

procedure TiOSNativeSpinBox.ValueChanged;
begin
  if Observers.IsObserving(TObserverMapping.EditLinkID) then
    if TLinkObservers.EditLinkEdit(Observers) then
      TLinkObservers.EditLinkModified(Observers);
  if Observers.IsObserving(TObserverMapping.ControlValueID) then
    TLinkObservers.ControlValueModified(Observers);

  Model.DisableNotify;
  try
    Model.Value := View.value;
  finally
    Model.EnableNotify;
  end;
  Model.Change;

  if Observers.IsObserving(TObserverMapping.EditLinkID) then
    TLinkObservers.EditLinkUpdate(Observers);
  if Observers.IsObserving(TObserverMapping.ControlValueID) then
    TLinkObservers.ControlValueUpdate(Observers);
end;

initialization
  TPresentationProxyFactory.Current.Register(TSpinBox, TControlType.Platform, TiOSSpinBoxProxy);
finalization
  TPresentationProxyFactory.Current.Unregister(TSpinBox, TControlType.Platform, TiOSSpinBoxProxy);
end.
